/*
 * the Decibel Realtime Communication Framework
 * Copyright (C) 2008 by George Goldberg
 *  @author George Goldberg <grundleborg@googlemail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "accountconnector.h"

#include <kwallet.h>
#include <kcomponentdata.h>

#include <QtCore/QStringList>
#include <QtCore/QSettings>
#include <QtCore/QHash>
#include <QtCore/QtDebug>
#include <QtCore/QObject>

/// @cond FALSE

AccountConnector::AccountConnector(QObject *parent) :
    AccountConnectorBase(parent),
    m_wallet(0),
    m_componentData(new KComponentData("kwalletintegration", "decibel")),
    m_walletOpen(false)
{ Q_ASSERT(0 != m_componentData); }

AccountConnector::~AccountConnector()
{
    closeStorage();
    delete m_componentData;
    delete m_wallet;
}

void AccountConnector::openStorage()
{
    // FIXME: error checking and window_id
    m_wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), 0,
                                           KWallet::Wallet::Asynchronous);
    Q_ASSERT(0 != m_wallet);
    connect(m_wallet, SIGNAL(walletOpened(bool)), this, SLOT(walletOpened(bool)));
    connect(m_wallet, SIGNAL(walletClosed()), this, SLOT(walletClosed()));
}

void AccountConnector::closeStorage()
{
    if (false == m_walletOpen) { return; }

    m_walletOpen = false;
    delete m_wallet;
    m_wallet = 0;
}

void AccountConnector::walletOpened(bool success)
{
    qDebug() << "AccountConnector.cpp: walletOpened(): Wallet Opened Slot Called. Success?:"
             << success;

    m_walletOpen = success;

    if (success) { emit accountDataAvailable(true); }
}

void AccountConnector::walletClosed()
{
    qDebug() << "AccountConnector.cpp: walletClosed():"
                " Wallet was closed.";
    m_walletOpen = false;
    emit accountDataAvailable(false);
}

bool AccountConnector::hasAccount(const uint id) const
{
    /*
     * first we must check that the wallet
     * is open and ready to receive stuff.
     */
    if(!m_walletOpen) { return false; }

    /*
     * the wallet is open, so we can look in
     * it to see if an account with the specified
     * id exists in it.
     */
    qDebug() << "AccountConnector::hasAccount(): looking for folder id:" << id;
    QString folderName;
    folderName.setNum(id);
    if(m_wallet->hasFolder(folderName))
        return true;
    else
        return false;
}

/*
 * add a new account to the wallet.
 */
uint AccountConnector::storeAccount(const QVariantMap & nv_pairs)
{
    /*
     * first we must check that the wallet is open.
     */
    if(!m_walletOpen) { return 0; }

    /*
     * now we need to find out the highest handle
     * that is free.
     */
    // FIXME: this is not the best way of choosing the handle... in fact its terrible :( !!!
    uint currentHandle = 0;
    QStringList folders = m_wallet->folderList();
    // FIXME: there are already 3 folders in our new wallet. this is insanely hack-tastic.....
    if(folders.size() <= 2)
    {
        /*
         * there are no folders, so start at 1.
         */
        qDebug() << "AccountConnector::storeAccount():"
                 << "there are no folders, so currentHandle=3";
        currentHandle = 2;
    }
    else
    {
        /*
         * go through all the handles used and
         * get the biggest.
         */
        qDebug() << "AccountConnector::storeAccount():"
                 << "there are folders in the wallet, so we need"
                    "to work out the currentHandle";
        QStringList::const_iterator end( folders.end() );
         // FIXME: this is horriffic!!! I can't believe I'm writing it!
        for ( QStringList::const_iterator itr( folders.begin() ); itr != end; ++itr )
        {
            qDebug() << "AccountConnector::storeAccount():"
                     << "iterating through folders: folder name: " << *itr;
             uint i = (*itr).toUInt();
             if(i > currentHandle)
                 currentHandle = i;
        }
        qDebug() << "AccountConnector::storeAccount():"
                 << "currentHandle: " << currentHandle;
    }

    /*
     * create the folder and then enter it
     */
    QString folder;
    folder.setNum((currentHandle+1));
    m_wallet->createFolder(folder); // FIXME: error checking
    m_wallet->setFolder(folder); // FIXME: error checking

    /*
     * now store the account data into the wallet.
     */
    QMap<QString, QVariant>::const_iterator end( nv_pairs.end() );
    for (QMap<QString, QVariant>::const_iterator itr(nv_pairs.begin());
            itr != end; ++itr )
    {
        QString key = itr.key();
        QVariant value = itr.value();

        if(value.canConvert(QVariant::ByteArray))
        {
            // FIXME: are all data passed as strings?
            m_wallet->writeEntry(key, value.toByteArray());
        }
        else if(value.canConvert(QVariant::Int))
        {
            QByteArray entry;
            entry.setNum(value.toInt());
            m_wallet->writeEntry(key, entry);
        }
        else
        {
            // FIXME: the type support is pretty mental. can we get the s:
            //        or p: or i: stuff from decibel in some way?
            qDebug() << "AccountConnector::storeAccount(): Unknown value type.";
        }
    }

    return (currentHandle+1);
}

void AccountConnector::updateAccount(const uint id, const QVariantMap & data)
{
    Q_ASSERT(hasAccount(id));

    /*
     * check that the wallet is open.
     */
    if(!m_walletOpen) { return; } // FIXME: do something about the error.

    /*
     * set the current wallet folder to the one of this
     * account.
     */
    // FIXME: user might have deleted the folder in kwallet
    QString folder;
    folder.setNum(id);
    m_wallet->setFolder(folder); // FIXME: error checking

    /*
     * iterate over the key/value pairs
     * and remove them from the wallet
     * if they are already there before
     * writing them to the wallet.
     */
    QMap<QString, QVariant>::const_iterator end( data.end() );
    for ( QMap<QString, QVariant>::const_iterator itr( data.begin() ); itr != end; ++itr )
    {
        QString key = itr.key();
        QVariant value = itr.value();

        if(m_wallet->hasEntry(key))
        {
            m_wallet->removeEntry(key); // FIXME: error checking
        }
        if(value.canConvert(QVariant::ByteArray))
        {
            m_wallet->writeEntry(key, value.toByteArray()); // FIXME: are all data passed as strings?
        }
        else if(value.canConvert(QVariant::Int))
        {
            QByteArray entry;
            entry.setNum(value.toInt());
            m_wallet->writeEntry(key, entry);
        }
        else
        {
            // FIXME: the type support is pretty mental. can we get the s: or p: or i: stuff from decibel in some way?
            qDebug() << "AccountConnector::updateAccount(): Unknown value type.";
        }
    }
}

QVariantMap AccountConnector::getAccount(const uint id) const
{
    if(!m_walletOpen) { return QVariantMap(); }

    QString folder;
    folder.setNum(id);
    m_wallet->setFolder(folder);
    QVariantMap kvp;
    QStringList entries = m_wallet->entryList();
    QStringList::const_iterator eend( entries.end() );
    for ( QStringList::const_iterator eitr( entries.begin() ); eitr != eend; ++eitr )
    {
        QByteArray value;
        m_wallet->readEntry((*eitr), value);

        qDebug() << "AccountConnector::getAccount():"
                 << "iterating over entries: key:" << (*eitr)
                 << "value:" << value;

        if(((*eitr) == Decibel::name_presence) )
           // || ((*eitr) == Decibel::name_current_presence)
        {
            //We have an integer
            uint np = value.toInt();
            kvp.insert((*eitr), np);

        }
        else
        {
            kvp.insert((*eitr), QString(value));
        }
    }
    return kvp;
}

bool AccountConnector::deleteAccount(const uint id)
{
    if(!m_walletOpen) { return false; }

    QString folder;
    folder.setNum(id);
    m_wallet->removeFolder(folder);    // FIXME: error checking.
    return true;
}

QList<uint> AccountConnector::accountIds() const
{
    if(!m_walletOpen) { return QList<uint>(); }

    qDebug() << "AccountConnector::accountIds(): started.";
    if(!m_wallet->isOpen())
        qDebug() << "AccountConnector::accountIds():"
                 << "Error: wallet is not open.";
    QList<uint> accounts;

    QStringList folders = m_wallet->folderList();
    QStringList::const_iterator end( folders.end() );
    for ( QStringList::const_iterator itr( folders.begin() ); itr != end; ++itr )
    {
        qDebug () << "AccountConnector::accountIds():"
                  << "Iterating over wallet folders: folder name:" << (*itr);
        bool ok;
        uint i = (*itr).toUInt(&ok);
        if(ok == false)
        {
            // FIXME: this is so the default folders are ignored otherwise
            //        Q_ASSERT failures
            qDebug() << "Ignoring wallet folder.";
        }
        else
        {
            qDebug() << "Folder name:" << i;
            accounts.append(i);
        }
    }

    return accounts;
}

QList<uint> AccountConnector::findAccounts(const QVariantMap & nv_pairs) const
{
    if(!m_walletOpen) { return QList<uint>(); }

    QList<uint> results;

    /*
     * loop through all the folders (accounts) in the
     * wallet.
     */
    QStringList folders = m_wallet->folderList();
    QStringList::const_iterator end( folders.end() );
    for ( QStringList::const_iterator itr( folders.begin() ); itr != end; ++itr )
    {
        /*
         * for each account we need to see if
         * all the nv_pairs parameters
         * match.
         */
        //uint i = (*itr).toUInt();
        //accounts.append(i);
        bool does_match(true);

        QString nv_key;
        foreach (nv_key, nv_pairs.keys())
        {
            QVariantMap accountData = getAccount((*itr).toUInt());
            if (!accountData.contains(nv_key) ||
                nv_pairs[nv_key].typeName() != accountData[nv_key].typeName() ||
                nv_pairs[nv_key] != accountData[nv_key])
            {
                does_match = false;
                break;
            }
        }
        if (does_match)
        {
            uint result = (*itr).toUInt();
            if(result != 0)
                results.append(result);
        }
    }
    return results;
}

bool AccountConnector::setValue(const uint id,
                                const QString & key, const QVariant & value)
{
    if(!m_walletOpen) { return false; }

    QString folder;
    folder.setNum(id);
    m_wallet->setFolder(folder);

    if(m_wallet->hasEntry(key))
    {
        m_wallet->removeEntry(key);    // FIXME: error checking
    }
    if(value.canConvert(QVariant::ByteArray))
    {
        m_wallet->writeEntry(key, value.toByteArray());    // FIXME: are all data passed as strings?
    }
    else if(value.canConvert(QVariant::Int))
    {
        QByteArray entry;
        entry.setNum(value.toInt());
        m_wallet->writeEntry(key, entry);
    }
    else
    {
        // FIXME: the type support is pretty mental. can we get the s: or p: or i:
        //        stuff from decibel in some way? does it even matter?
        qDebug() << "AccountConnector::updateAccount(): Unknown value type.";
    }
    return true;    // FIXME: error checking
}

/// @endcond
