/*
 *   Copyright 2007,2008 by Alex Merry <alex.merry@kdemail.net>
 *
 *   Some code (text size calculation) taken from clock applet:
 *   Copyright 2007 by Sebastian Kuegler <sebas@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .
 */

#include "nowplaying.h"

#include <QPainter>

#include <KGlobalSettings>
#include <Plasma/Theme>

NowPlaying::NowPlaying(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_state(NoPlayer),
      m_track(0),
      m_length(0),
      m_position(0),
      m_font(KGlobalSettings::generalFont()),
      m_fontDirty(true)
{
    setContentSize(QSize(200, 60));
    setMinimumContentSize(QSizeF(100, 30));

    m_leftColText = i18nc("For a song or other music", "Artist") + ":   \n" +
                    i18nc("For a song or other music", "Title") + ":   \n" +
                    i18nc("For a song or other music", "Album") + ":   \n" +
                    i18nc("Position in a song", "Time") + ":   ";
}

NowPlaying::~NowPlaying()
{
}

void NowPlaying::init()
{
    dataEngine("nowplaying")->connectSource("Players", this, 5000);
    connect(this, SIGNAL(geometryChanged()), this, SLOT(sizeChanged()));
}

void NowPlaying::sizeChanged()
{
    m_fontDirty = true;
}

void NowPlaying::paintInterface(QPainter *p,
                                const QStyleOptionGraphicsItem *option,
                                const QRect &contentsRect)
{
    Q_UNUSED(option)

    if (m_fontDirty) {
        // Choose a relatively big font size to start with and decrease it from there to fit.
        m_font.setPointSize(qMax((int)(contentSize().height()/1.5), 1));
        p->setFont(m_font);
        QRect textRect = p->boundingRect(contentsRect, QPainter::TextAntialiasing, m_leftColText);
        while ((textRect.width() > contentsRect.width() || textRect.height() > contentsRect.height()) && m_font.pointSize() > 1) {
            m_font.setPointSize(m_font.pointSize() - 1);
            p->setFont(m_font);
            textRect = p->boundingRect(contentsRect, QPainter::TextAntialiasing, m_leftColText);
        }
    }

    // make sure we can read it on the theme's background
    p->setPen(QPen(Plasma::Theme::self()->textColor()));
    p->setFont(m_font);

    if (m_state == NoPlayer) {
        p->drawText(contentsRect,
                    Qt::AlignCenter,
                    i18n("No media player found"));
    } else if (m_state == Stopped) {
        p->drawText(contentsRect,
                    Qt::AlignCenter,
                    i18nc("The state of a music player", "Stopped"));
    } else {
        int leftColWidth = p->boundingRect(contentsRect,
                                           Qt::AlignLeft | QPainter::TextAntialiasing,
                                           m_leftColText).width();
        QRect rightColRect(contentsRect.adjusted(leftColWidth, 0, 0, 0));
        int rightColWidth = rightColRect.width();

        QFontMetrics metrics = p->fontMetrics();
        QString display = metrics.elidedText(m_artist, Qt::ElideRight, rightColWidth) + '\n' +
                          metrics.elidedText(m_title, Qt::ElideRight, rightColWidth) + '\n' +
                          metrics.elidedText(m_album, Qt::ElideRight, rightColWidth) + '\n';

        if (m_length != 0) {
            QString time = QString::number(m_position / 60) + ':' +
                           QString::number(m_position % 60).rightJustified(2, '0') + " / " +
                           QString::number(m_length / 60) + ':' +
                           QString::number(m_length % 60).rightJustified(2, '0');
            display += metrics.elidedText(time, Qt::ElideRight, rightColWidth);
        }

        p->drawText(contentsRect,
                    Qt::AlignLeft,
                    m_leftColText);
        p->drawText(rightColRect,
                    Qt::AlignLeft,
                    display);
    }
}

void NowPlaying::constraintsUpdated(Plasma::Constraints)
{
    // we should probably change to everything in one line if we're horizonal
    updateGeometry();
}

void NowPlaying::dataUpdated(const QString &name,
                             const Plasma::DataEngine::Data &data)
{
    if (name == "Players") {
        if (data.isEmpty()) {
            if (!m_watchingPlayer.isEmpty()) {
                dataEngine("nowplaying")->disconnectSource(m_watchingPlayer, this);
                m_state = NoPlayer;
            }
            m_watchingPlayer.clear();
        } else if (!data.contains(m_watchingPlayer)) {
            if (!m_watchingPlayer.isEmpty()) {
                dataEngine("nowplaying")->disconnectSource(m_watchingPlayer, this);
                m_state = NoPlayer;
            }
            // just pick one - we should be more intelligent, like picking
            // one that's actually playing
            m_watchingPlayer = data.values().first().toString();
            dataEngine("nowplaying")->connectSource(m_watchingPlayer, this, 1000);
        }
    } else {
        if (data.isEmpty()) {
            m_state = NoPlayer;
            return;
        }
        if (data["State"].toString() == "playing") {
            m_state = Playing;
        } else if (data["State"].toString() == "paused") {
            m_state = Paused;
        } else {
            m_state = Stopped;
        }
        m_artist = data["Artist"].toString();
        m_album = data["Album"].toString();
        m_title = data["Title"].toString();
        m_track = data["Track"].toInt();
        //m_comment = data["Comment"].toString();
        //m_genre = data["Genre"].toString();
        m_length = data["Length"].toInt();
        m_position = data["Position"].toInt();
        //m_artwork = data["Artwork"].value<QPixmap>();
        update();
    }
}

#include "nowplaying.moc"
