/*
 *   Copyright 2007 Alex Merry <alex.merry@kdemail.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "mpris.h"
#include <mpris_interface.h>

#include <QtDBus>
#include <QFile>

#include <kdebug.h>
#include <KIO/NetAccess>

MprisWatcher::MprisWatcher(QObject* parent)
    : QObject(parent),
      sessionBus(0)
{
    QDBusConnection sessionCon = QDBusConnection::sessionBus();
    if (sessionCon.isConnected()) {
        sessionBus = sessionCon.interface();
        QDBusReply<QStringList> reply = sessionBus->registeredServiceNames();
        if (reply.isValid()) {
            QStringList services = reply.value().filter("org.mpris.");
            foreach (QString serviceName, services) {
                players << serviceName.replace("org.mpris.", QString());
            }
            kDebug() << "Players now" << players;
        } else {
            kWarning() << "Couldn't get service names:" << reply.error().message();
        }
        connect(sessionBus, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
                this,       SLOT(serviceChange(QString,QString,QString)));
    } else {
        kWarning() << "Couldn't connect to session bus";
    }
}

MprisWatcher::~MprisWatcher()
{
}

QStringList MprisWatcher::getPlayers()
{
    return players;
}

void MprisWatcher::serviceChange(const QString& serviceName,
                                 const QString& oldOwner,
                                 const QString& newOwner)
{
    if (serviceName.startsWith("org.mpris.")) {
        QString name(serviceName);
        name.replace("org.mpris.", QString());
        if (oldOwner.isEmpty()) {
            // got a new service
            players << name;
            emit(newPlayer(name));
        } else if (newOwner.isEmpty()) {
            // an old service disappeared
            players.removeAll(name);
            emit(playerDisappeared(name));
        }
    }
}






Mpris::Mpris(const QString& name, QObject* parent)
    : QObject(parent),
      player(0),
      m_playerName(name)
{
    setup();
}

Mpris::~Mpris()
{
    delete player;
}

void Mpris::setup() {
    delete player;
    player = new OrgFreedesktopMediaPlayerInterface(
            QString("org.mpris.").append(m_playerName),
            "/Player",
            QDBusConnection::sessionBus());

    m_metadata.clear();
    m_state = Stopped;
    m_caps = NO_CAPS;

    if (player->isValid()) {
        connect(player, SIGNAL(CapsChange(int)),
                this,   SLOT(capsChanged(int)));
        connect(player, SIGNAL(TrackChange(QVariantMap)),
                this,   SLOT(trackChanged(QVariantMap)));
        connect(player, SIGNAL(StatusChange(int)),
                this,   SLOT(stateChanged(int)));

        capsChanged(player->GetCaps());
        trackChanged(player->GetMetadata());
        stateChanged(player->GetStatus());
    }
}

QString Mpris::name()
{
    return m_playerName;
}

bool Mpris::isRunning()
{
    if (!player->isValid()) {
        setup();
    }
    return player->isValid();
}

Player::State Mpris::state()
{
    return m_state;
}

QString Mpris::artist()
{
    if (m_metadata.contains("artist")) {
        return m_metadata["artist"].toString();
    }
    return QString();
}

QString Mpris::album()
{
    if (m_metadata.contains("album")) {
        return m_metadata["album"].toString();
    }
    return QString();
}

QString Mpris::title()
{
    if (m_metadata.contains("title")) {
        return m_metadata["title"].toString();
    }
    return QString();
}

int Mpris::trackNumber()
{
    if (m_metadata.contains("trackNumber")) {
        QString track = m_metadata["trackNumber"].toString();
        int pos = track.indexOf('/');
        if (pos >= 0) {
            track.truncate(pos);
        }
        return track.toInt();
    }
    return 0;
}

QString Mpris::comment()
{
    if (m_metadata.contains("comment")) {
        return m_metadata["comment"].toString();
    }
    return QString();
}

QString Mpris::genre()
{
    if (m_metadata.contains("genre")) {
        return m_metadata["genre"].toString();
    }
    return QString();
}

int Mpris::length()
{
    if (m_metadata.contains("time")) {
        return m_metadata["time"].toInt();
    } else if (m_metadata.contains("length")) {
        // stupid audacious...
        return m_metadata["length"].toInt()/1000;
    }
    return 0;
}

int Mpris::position()
{
    if (player->isValid()) {
        return player->PositionGet() / 1000;
    }
    return 0;
}

float Mpris::volume()
{
    if (player->isValid()) {
        return ((float)player->VolumeGet()) / 100;
    }
    return 0;
}

QPixmap Mpris::artwork()
{
    if (m_metadata.contains("arturl")) {
        QString arturl = m_metadata["arturl"].toString();
        if (!arturl.isEmpty()) {
            if (!artfiles.contains(arturl) ||
                !QFile::exists(artfiles[arturl])) {
                QString artfile;
                if (!KIO::NetAccess::download(arturl, artfile, 0)) {
                    kWarning() << KIO::NetAccess::lastErrorString();
                    return QPixmap();
                }
                artfiles[arturl] = artfile;
            }
            return QPixmap(artfiles[arturl]);
        }
    }
    return QPixmap();
}

bool Mpris::canPlay()
{
    return m_caps & CAN_PLAY;
}

void Mpris::play()
{
    if (player->isValid()) {
        player->Play();
    }
}

bool Mpris::canPause()
{
    return m_caps & CAN_PAUSE;
}

void Mpris::pause()
{
    if (player->isValid()) {
        player->Pause();
    }
}

bool Mpris::canStop()
{
    return m_state != Stopped;
}

void Mpris::stop()
{
    if (player->isValid()) {
        player->Stop();
    }
}

bool Mpris::canGoPrevious()
{
    return m_caps & CAN_GO_PREV;
}

void Mpris::previous()
{
    if (player->isValid()) {
        player->Prev();
    }
}

bool Mpris::canGoNext()
{
    return m_caps & CAN_GO_NEXT;
}

void Mpris::next()
{
    if (player->isValid()) {
        player->Next();
    }
}

bool Mpris::canSetVolume()
{
    return true;
}

void Mpris::setVolume(float volume)
{
    if (player->isValid()) {
        player->VolumeSet(volume * 100);
    }
}

bool Mpris::canSeek()
{
    return m_caps & CAN_SEEK;
}

void Mpris::seek(int time)
{
    if (player->isValid()) {
        // player->seek() wants milliseconds
        player->PositionSet(time * 1000);
    }
}

// SLOTS

void Mpris::trackChanged(const QVariantMap& metadata)
{
    kDebug() << m_playerName << "metadata:" << metadata;
    m_metadata = metadata;
}

void Mpris::stateChanged(int state)
{
    kDebug() << m_playerName << "state:" << state;
    switch (state) {
        case 0:
            m_state = Playing;
            break;
        case 1:
            m_state = Paused;
            break;
        case 2:
            m_state = Stopped;
            break;
        default:
            kDebug() << m_playerName << "unexpected state" << state;
    }
}

void Mpris::capsChanged(int caps)
{
    kDebug() << m_playerName << "capabilities:" << caps;
    m_caps = static_cast<Caps>(caps);
    if (!caps & CAN_PROVIDE_METADATA) {
        m_metadata.clear();
    }
}


#include "mpris.moc"
