/*
 *   Copyright 2007 Alex Merry <alex.merry@kdemail.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MPRIS_H
#define MPRIS_H

#include "player.h"

#include <QObject>
#include <QVariantMap>
#include <QStringList>

class OrgFreedesktopMediaPlayerInterface;
class QDBusConnectionInterface;

class MprisWatcher : public QObject
{
    Q_OBJECT

public:
    MprisWatcher(QObject* parent = 0);
    ~MprisWatcher();

    QStringList getPlayers();

Q_SIGNALS:
    /**
     * A new player appeared on D-Bus
     *
     * @param the name, as appended to org.mpris.
     */
    void newPlayer(const QString& name);
    /**
     * A player disappeared from D-Bus
     *
     * @param the name, as appended to org.mpris.
     */
    void playerDisappeared(const QString& name);

private Q_SLOTS:
    void serviceChange(const QString& name,
                       const QString& oldOwner,
                       const QString& newOwner);

private:
    QStringList players;
    QDBusConnectionInterface* sessionBus;
};

/**
 * Allows getting info from players supporting the MPRIS interface
 */
class Mpris : public QObject, public Player
{
    Q_OBJECT

public:
    /**
     * @param name     the media player name.
     */
    explicit Mpris(const QString& name, QObject* parent = 0);
    ~Mpris();

    QString name();
    bool isRunning();
    State state();
    QString artist();
    QString album();
    QString title();
    int trackNumber();
    QString comment();
    QString genre();
    int length();
    int position();
    float volume();
    QPixmap artwork();

    bool canPlay();
    void play();
    bool canPause();
    void pause();
    bool canStop();
    void stop();
    bool canGoPrevious();
    void previous();
    bool canGoNext();
    void next();

    bool canSetVolume();
    void setVolume(float volume);

    bool canSeek();
    void seek(int time);

private Q_SLOTS:
    void trackChanged(const QVariantMap& metadata);
    void stateChanged(int state);
    void capsChanged(int caps);

private:
    enum Caps {
        NO_CAPS               = 0,
        CAN_GO_NEXT           = 1,
        CAN_GO_PREV           = 2,
        CAN_PAUSE             = 4,
        CAN_PLAY              = 8,
        CAN_SEEK              = 16,
        CAN_PROVIDE_METADATA  = 32
    };
    void setup();
    OrgFreedesktopMediaPlayerInterface* player;

    QString m_playerName;
    QVariantMap m_metadata;
    State m_state;
    Caps m_caps;
    QMap<QString,QString> artfiles;
};

#endif // MPRIS_H
