/*
 *   Copyright 2007 Alex Merry <alex.merry@kdemail.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "nowplayingengine.h"

#include <config-nowplaying.h>

#include <QStringList>

#include <KDebug>
#include <KLocale>

#include "player.h"
#include "mpris.h"
#ifdef JUK_FOUND
#include "juk.h"
#endif // JUK_FOUND
#ifdef XMMS_FOUND
#include "xmms.h"
#endif // XMMS_FOUND

NowPlayingEngine::NowPlayingEngine(QObject* parent,
                                   const QVariantList& args)
    : Plasma::DataEngine(parent),
      mprisWatcher(new MprisWatcher(this))
{
    Q_UNUSED(args)

    // For some reason, this screws everything up
    //setMinimumUpdateInterval(333);

#ifdef JUK_FOUND
    addPlayer(new Juk(this));
#endif
#ifdef XMMS_FOUND
    addPlayer(new Xmms(this));
#endif

    foreach (QString player, mprisWatcher->getPlayers()) {
        mprisAdd(player);
    }
    connect(mprisWatcher, SIGNAL(newPlayer(QString)),
            this,         SLOT(mprisAdd(QString)));
    connect(mprisWatcher, SIGNAL(playerDisappeared(QString)),
            this,         SLOT(mprisRemove(QString)));
}

NowPlayingEngine::~NowPlayingEngine()
{
}

bool NowPlayingEngine::sourceRequested(const QString &source)
{
    QString lowerSource = source.toLower();
    if (lowerSource == "help") {
        setData(source, "Use 'players' to get a list of players.\n"
                        "Use 'properties' to get a list of all properties that may be returned.\n"
                        "Players can be controlled with\n"
                        "    playername:command[:args]\n"
                        "eg: request the source 'JuK:pause' to pause JuK, or 'JuK:volume:0.6' to set the volume to 60%\n"
                        "Note that player names are case sensitive\n\n"

                        "Available commands:\n"
                        "    play     - play the currently selected track\n"
                        "    pause    - pause the currently playing track (may work as play/pause depending on player)\n"
                        "    stop     - stop the currently playing track\n"
                        "    next     - move to the next track in the playlist\n"
                        "    previous - move to the previous track in the playlist\n"
                        "    volume   - set the volume, takes one argument: a volume between 0 and 1\n"
                        "    seek     - seek to a given point in the track, takes one argument: the position in seconds");
        return true;
    } else if (lowerSource == "properties") {
        setData(source, "State",           "QString - playing|paused|stopped");
        setData(source, "Artist",          "QString - the artist metadata for the\n"
                                           "          current track, if available");
        setData(source, "Album",           "QString - the album metadata for the\n"
                                           "          current track, if available");
        setData(source, "Title",           "QString - the title metadata for the\n"
                                           "          current track, if available");
        setData(source, "Track number",    "int     - the album/collection track number\n"
                                           "          (eg: on a CD) if known, 0 otherwise");
        setData(source, "Comment",         "QString - the comment metadata for the\n"
                                           "          current track, if available");
        setData(source, "Genre",           "QString - the comment metadata for the\n"
                                           "          current track, if available");
        setData(source, "Length",          "int     - the length of the current track\n"
                                           "          in seconds, 0 if unknown");
        setData(source, "Position",        "int     - the position of the current track\n"
                                           "          in seconds, 0 if unknown");
        setData(source, "Volume",          "float   - the volume, given as a float\n"
                                           "          between 0 and 1, or -1 if unknown");
        setData(source, "Artwork",         "QPixmap - the album artwork, if available");
        setData(source, "Can play",        "bool    - whether the 'play' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can pause",       "bool    - whether the 'pause' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can stop",        "bool    - whether the 'stop' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can go previous", "bool    - whether the 'previous' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can go next",     "bool    - whether the 'next' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can set volume",  "bool    - whether the 'volume' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        setData(source, "Can seek",        "bool    - whether the 'seek' command can be\n"
                                           "          expected to have any effect\n"
                                           "          (nb: not definitive - if this is\n"
                                           "          true, the command may still fail)");
        return true;
    } else if (lowerSource == "players") {
        QStringList activePlayers;
        foreach (Player* player, players) {
            if (player->isRunning()) {
                activePlayers << player->name();
            }
        }
        setData(source, activePlayers);
        return true;
    } else if (source.contains(':')) {
        // A command - we always return false here
        QStringList command = source.split(':');

        kDebug() << "Command is" << command;

        if (players.contains(command[0]) && command.count() >= 2) {
            Player* player = players[command[0]];
            Q_ASSERT(player);
            QString commandName = command[1].toLower();

            if (commandName == "play") {
                player->play();
            } else if (commandName == "pause") {
                player->pause();
            } else if (commandName == "stop") {
                player->stop();
            } else if (commandName == "previous") {
                player->previous();
            } else if (commandName == "next") {
                player->next();
            } else if (commandName == "volume") {
                if (command.count() >= 3) {
                    bool ok = false;
                    float volume = command[2].toFloat(&ok);
                    if (ok && volume >= 0 && volume <= 1) {
                        player->setVolume(volume);
                    } else {
                        kWarning() << command[2] << "is not a valid volume";
                    }
                } else {
                    kWarning() << "Not enough arguments to \"volume\"";
                }
            } else if (commandName == "seek") {
                if (command.count() >= 3) {
                    bool ok = false;
                    int pos = command[2].toInt(&ok);
                    if (ok && pos >= 0 && pos <= player->length()) {
                        player->seek(pos);
                    } else {
                        kWarning() << command[2] << "is not a valid position";
                    }
                } else {
                    kWarning() << "Not enough arguments \"to seek\"";
                }
            } else {
                kWarning() << "Invalid command" << commandName;
            }
        } else {
            kWarning() << "Player" << command[0] << "not found";
        }

        return false;
    } else if (players.contains(source)) {
        return updateSource(source);
    } else {
        kWarning() << "Player" << source << "not found";
    }

    return false;
}

bool NowPlayingEngine::updateSource(const QString& source)
{
    QString lowerSource = source.toLower();
    if (lowerSource == "help" || lowerSource == "properties") {
        // help text doesn't change
        return true;
    } else if (lowerSource == "players") {
        QStringList activePlayers;
        foreach (Player* player, players) {
            if (player->isRunning()) {
                activePlayers << player->name();
            }
        }
        setData(source, activePlayers);
        return true;
    }

    if (!players.contains(source)) {
        // maybe an MPRIS player went?
        clearData(source);
        return false;
    }
    Player* player = players[source];
    Q_ASSERT(player);

    if (!player->isRunning()) {
        clearData(source);
        return false;
    }

    switch(player->state()) {
        case Player::Playing:
            setData(source, "State", "playing");
            break;
        case Player::Paused:
            setData(source, "State", "paused");
            break;
        case Player::Stopped:
            setData(source, "State", "stopped");
            break;
    }

    setData(source, "Artist", player->artist());
    setData(source, "Album", player->album());
    setData(source, "Title", player->title());
    setData(source, "Track number", player->trackNumber());
    setData(source, "Comment", player->comment());
    setData(source, "Genre", player->genre());
    setData(source, "Length", player->length());
    setData(source, "Position", player->position());
    setData(source, "Volume", player->volume());
    setData(source, "Artwork", player->artwork());
    setData(source, "Can play", player->canPlay());
    setData(source, "Can pause", player->canPause());
    setData(source, "Can stop", player->canStop());
    setData(source, "Can go previous", player->canGoPrevious());
    setData(source, "Can go next", player->canGoNext());
    setData(source, "Can set volume", player->canSetVolume());
    setData(source, "Can seek", player->canSeek());

    return false;
}

void NowPlayingEngine::mprisAdd(const QString& name)
{
    kDebug() << "New MPRIS player found:" << name;
    addPlayer(new Mpris(name, this));
}

void NowPlayingEngine::mprisRemove(const QString& name)
{
    kDebug() << "MPRIS player" << name << "disappeared";
    if (players.contains(name)) {
        Player *removed = players.take(name);
        delete removed;
    } else {
        kDebug() << "We didn't know about MPRIS player" << name;
    }
}

void NowPlayingEngine::addPlayer(Player* player)
{
    kDebug() << "Adding player" << player->name();
    players.insert(player->name(), player);
}

#include "nowplayingengine.moc"
