/*
 *   Copyright 2007 Alex Merry <alex.merry@kdemail.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef PLAYERINFO_H
#define PLAYERINFO_H

#include <QString>
#include <QPixmap>

class Player
{
public:
    virtual ~Player() {}

    /**
     * The name of this player
     */
    virtual QString name() = 0;
    /**
     * Current state of the player
     */
    enum State {
        Playing,
        Paused,
        Stopped
    };
    /**
     * Whether the player is running and accessible
     */
    virtual bool isRunning() = 0;
    /**
     * Current state of the player
     *
     * Undefined if !running()
     *
     * See State
     */
    virtual State state() = 0;
    /** Artist for current track.  May be empty */
    virtual QString artist() { return QString(); }
    /** Album for current track.  May be empty */
    virtual QString album() { return QString(); }
    /** Title of current track.  May be empty */
    virtual QString title() { return QString(); }
    /**
     * Track number of current track.
     *
     * Note that this is the track on a CD or in an album, not
     * the playlist position
     *
     * 0 if no track defined, or unknown.
     */
    virtual int trackNumber() { return 0; }
    /** Comment for current track.  May be empty */
    virtual QString comment() { return QString(); }
    /** Genre of current track.  May be empty */
    virtual QString genre() { return QString(); }
    /**
     * Length of current track in seconds.
     *
     * 0 if unknown.
     */
    virtual int length() { return 0; }
    /**
     * Position of current track in seconds.
     *
     * 0 if unknown, not defined if Stopped.
     */
    virtual int position() { return 0; }
    /**
     * Current volume
     *
     * Value should be between 0 and 1
     *
     * -1 if unknown
     */
    virtual float volume() { return -1; }
    /**
     * Album artwork
     *
     * Null (pm.isNull()) if none available
     */
    virtual QPixmap artwork() { return QPixmap(); }

    // Commands

    /**
     * Play a track
     */
    virtual bool canPlay() { return false; }
    virtual void play() {}
    /**
     * Pause the currently playing track
     */
    virtual bool canPause() { return false; }
    virtual void pause() {}
    /**
     * Stop the currently playing track
     *
     * canStop() should usually be state() != Stopped if
     * stop() is implemented
     */
    virtual bool canStop() { return false; }
    virtual void stop() {}
    /**
     * Move to the previous track
     */
    virtual bool canGoPrevious() { return false; }
    virtual void previous() {}
    /**
     * Move to the next track
     */
    virtual bool canGoNext() { return false; }
    virtual void next() {}

    /**
     * Set the volume
     *
     * Must be between 0 and 1
     */
    virtual bool canSetVolume() { return false; }
    virtual void setVolume(float volume) { Q_UNUSED(volume) }

    /**
     * Set the position (in seconds)
     *
     * Should be <= length()
     */
    virtual bool canSeek() { return false; }
    virtual void seek(int time) { Q_UNUSED(time) }
};

#endif // PLAYERINFO_H
